@@ -11,19 +11,20 @@
 
 package MCE::Core::Manager;
 
-our $VERSION = '1.499_005'; $VERSION = eval $VERSION;
+use strict;
+use warnings;
+
+our $VERSION = '1.698_001';
+
+## no critic (TestingAndDebugging::ProhibitNoStrict)
 
 ## Items below are folded into MCE.
 
 package MCE;
 
-use strict;
-use warnings;
+no warnings qw( threads recursion uninitialized );
 
-## Warnings are disabled to minimize bits of noise when user or OS signals
-## the script to exit. e.g. MCE_script.pl < infile | head
-
-no warnings 'threads'; no warnings 'uninitialized';
+use bytes;
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -33,7 +34,9 @@ no warnings 'threads'; no warnings 'uninitialized';
 
 sub _task_end {
 
-   my MCE $self = $_[0]; my $_task_id = $_[1];
+   my ($self, $_task_id) = @_;
+
+   @_ = ();
 
    if (defined $self->{user_tasks}) {
       my $_task_end = (exists $self->{user_tasks}->[$_task_id]->{task_end})
@@ -63,25 +66,69 @@ sub _task_end {
 
 sub _output_loop {
 
-   my MCE $self = $_[0]; my $_input_data = $_[1]; my $_input_glob = $_[2];
-
-   my $_plugin_function   = $_[3];
-   my $_plugin_loop_begin = $_[4];
-   my $_plugin_loop_end   = $_[5];
+   my ( $self, $_input_data, $_input_glob, $_plugin_function,
+        $_plugin_loop_begin, $_plugin_loop_end ) = @_;
 
    @_ = ();
 
-   die "Private method called" unless (caller)[0]->isa( ref($self) );
+   my (
+      $_aborted, $_eof_flag, $_max_retries, $_syn_flag, %_sendto_fhs,
+      $_cb, $_chunk_id, $_chunk_size, $_fd, $_file, $_flush_file, $_wa,
+      @_is_c_ref, @_is_h_ref, @_is_q_ref, $_on_post_exit, $_on_post_run,
+      $_has_user_tasks, $_sess_dir, $_task_id, $_user_error, $_user_output,
+      $_input_size, $_offset_pos, $_single_dim, @_gather, $_cs_one_flag,
+      $_exit_id, $_exit_pid, $_exit_status, $_exit_wid, $_len, $_sync_cnt,
+      $_BSB_W_SOCK, $_BSE_W_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR,
+      $_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT
+   );
 
-   my ($_aborted, $_eof_flag, $_syn_flag, %_sendto_fhs, $_value, $_want_id);
-   my ($_callback, $_chunk_id, $_chunk_size, $_fd, $_file, $_flush_file);
-   my (@_gather, @_is_c_ref, @_is_q_ref, $_on_post_exit, $_on_post_run);
-   my ($_has_user_tasks, $_sess_dir, $_task_id, $_user_error, $_user_output);
-   my ($_input_size, $_len, $_offset_pos, $_single_dim, $_use_slurpio);
-   my ($_exit_id, $_exit_pid, $_exit_status, $_exit_wid, $_sync_cnt);
+   ## -------------------------------------------------------------------------
+   ## Callback return.
 
-   my ($_BSB_W_SOCK, $_BSE_W_SOCK, $_DAT_R_SOCK, $_DAU_R_SOCK, $_MCE_STDERR);
-   my ($_I_FLG, $_O_FLG, $_I_SEP, $_O_SEP, $_RS, $_RS_FLG, $_MCE_STDOUT);
+   my $_cb_ret_a = sub {                          ## CBK return array
+
+      my $_buf = $self->{freeze}($_[0]);
+         $_len = length $_buf; local $\ = undef if (defined $\);
+
+      if ($_len < FAST_SEND_SIZE) {
+         print {$_DAU_R_SOCK} $_len . $LF . $_buf;
+      } else {
+         print {$_DAU_R_SOCK} $_len . $LF;
+         print {$_DAU_R_SOCK} $_buf;
+      }
+
+      return;
+   };
+
+   my $_cb_ret_r = sub {                          ## CBK return reference
+
+      my $_buf = $self->{freeze}($_[0]);
+         $_len = length $_buf; local $\ = undef if (defined $\);
+
+      if ($_len < FAST_SEND_SIZE) {
+         print {$_DAU_R_SOCK} WANTS_REF . $LF . $_len . $LF . $_buf;
+      } else {
+         print {$_DAU_R_SOCK} WANTS_REF . $LF . $_len . $LF;
+         print {$_DAU_R_SOCK} $_buf;
+      }
+
+      return;
+   };
+
+   my $_cb_ret_s = sub {                          ## CBK return scalar
+
+      $_len = (defined $_[0]) ? length $_[0] : -1;
+      local $\ = undef if (defined $\);
+
+      if ($_len < FAST_SEND_SIZE) {
+         print {$_DAU_R_SOCK} WANTS_SCALAR . $LF . $_len . $LF . $_[0];
+      } else {
+         print {$_DAU_R_SOCK} WANTS_SCALAR . $LF . $_len . $LF;
+         print {$_DAU_R_SOCK} $_[0];
+      }
+
+      return;
+   };
 
    ## -------------------------------------------------------------------------
    ## Create hash structure containing various output functions.
@@ -142,7 +189,7 @@ sub _output_loop {
             }
          }
 
-         my $_exit_msg = '';
+         my ($_exit_msg, $_retry_buf) = ('', '');
 
          chomp($_exit_wid    = <$_DAU_R_SOCK>);
          chomp($_exit_pid    = <$_DAU_R_SOCK>);
@@ -152,14 +199,19 @@ sub _output_loop {
 
          read($_DAU_R_SOCK, $_exit_msg, $_len) if ($_len);
 
-         $self->{_wrk_status} = $_exit_status
-            if (abs($_exit_status) > abs($self->{_wrk_status}));
+         chomp($_len = <$_DAU_R_SOCK>);
+
+         read($_DAU_R_SOCK, $_retry_buf, $_len) if ($_len);
+
+         if (abs($_exit_status) > abs($self->{_wrk_status})) {
+            $self->{_wrk_status} = $_exit_status;
+         }
 
          ## Reap child/thread. Note: Win32 uses negative PIDs.
 
          if ($_exit_pid =~ /^PID_(-?\d+)/) {
             my $_pid = $1; my $_list = $self->{_pids};
-            for my $i (0 .. @$_list) {
+            for my $i (0 .. @{ $_list }) {
                if ($_list->[$i] && $_list->[$i] == $_pid) {
                   waitpid $_pid, 0;
                   $self->{_pids}->[$i] = undef;
@@ -169,7 +221,7 @@ sub _output_loop {
          }
          elsif ($_exit_pid =~ /^TID_(\d+)/) {
             my $_tid = $1; my $_list = $self->{_tids};
-            for my $i (0 .. @$_list) {
+            for my $i (0 .. @{ $_list }) {
                if ($_list->[$i] && $_list->[$i] == $_tid) {
                   ${ $self->{_thrs}->[$i] }->join();
                   $self->{_thrs}->[$i] = undef;
@@ -185,10 +237,23 @@ sub _output_loop {
          if (defined $_on_post_exit) {
             $self->{_exited_wid} = $_exit_wid;
 
-            $_on_post_exit->($self, {
-               wid => $_exit_wid, pid => $_exit_pid, status => $_exit_status,
-               msg => $_exit_msg, id  => $_exit_id
-            });
+            if (length($_retry_buf)) {
+               $self->{_retry} = $self->{thaw}($_retry_buf);
+               my $_retry_cnt  = $_max_retries - $self->{_retry}[2] - 1;
+
+               $_on_post_exit->($self, {
+                  wid => $_exit_wid, pid => $_exit_pid, status => $_exit_status,
+                  msg => $_exit_msg, id  => $_exit_id
+               }, $_retry_cnt);
+
+               delete $self->{_retry};
+            }
+            else {
+               $_on_post_exit->($self, {
+                  wid => $_exit_wid, pid => $_exit_pid, status => $_exit_status,
+                  msg => $_exit_msg, id  => $_exit_id
+               }, $_max_retries || 0 );
+            }
 
             delete $self->{_exited_wid};
          }
@@ -207,32 +272,38 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_A_ARY.$LF => sub {                   ## Array << Array
-         my $_buffer;
+         my $_buf;
 
          if ($_offset_pos >= $_input_size || $_aborted) {
             local $\ = undef if (defined $\);
-            print $_DAU_R_SOCK '0' . $LF;
+            print {$_DAU_R_SOCK} '0' . $LF;
             return;
          }
 
-         if ($_single_dim && $_chunk_size == 1) {
-            $_buffer = $_input_data->[$_offset_pos];
+         if ($_single_dim && $_cs_one_flag) {
+            $_buf = $_input_data->[$_offset_pos];
          }
          else {
             if ($_offset_pos + $_chunk_size - 1 < $_input_size) {
-               $_buffer = $self->{freeze}( [ @$_input_data[
+               $_buf = $self->{freeze}( [ @{ $_input_data }[
                   $_offset_pos .. $_offset_pos + $_chunk_size - 1
                ] ] );
             }
             else {
-               $_buffer = $self->{freeze}( [ @$_input_data[
+               $_buf = $self->{freeze}( [ @{ $_input_data }[
                   $_offset_pos .. $_input_size - 1
                ] ] );
             }
          }
 
-         local $\ = undef if (defined $\); $_len = length $_buffer;
-         print $_DAU_R_SOCK $_len . $LF . (++$_chunk_id) . $LF . $_buffer;
+         $_len = length $_buf; local $\ = undef if (defined $\);
+
+         if ($_len < FAST_SEND_SIZE) {
+            print {$_DAU_R_SOCK} $_len . $LF . (++$_chunk_id) . $LF . $_buf;
+         } else {
+            print {$_DAU_R_SOCK} $_len . $LF . (++$_chunk_id) . $LF;
+            print {$_DAU_R_SOCK} $_buf;
+         }
 
          $_offset_pos += $_chunk_size;
 
@@ -240,7 +311,7 @@ sub _output_loop {
       },
 
       OUTPUT_S_GLB.$LF => sub {                   ## Scalar << Glob FH
-         my $_buffer;
+         my $_buf = '';
 
          ## The logic below honors ('Ctrl/Z' in Windows, 'Ctrl/D' in Unix)
          ## when reading from standard input. No output will be lost as
@@ -248,7 +319,7 @@ sub _output_loop {
 
          if ($_eof_flag || $_aborted) {
             local $\ = undef if (defined $\);
-            print $_DAU_R_SOCK '0' . $LF;
+            print {$_DAU_R_SOCK} '0' . $LF;
             return;
          }
 
@@ -256,29 +327,93 @@ sub _output_loop {
             local $/ = $_RS if ($_RS_FLG);
 
             if ($_chunk_size <= MAX_RECS_SIZE) {
-               for (1 .. $_chunk_size) {
-                  if (defined($_ = <$_input_glob>)) {
-                     $_buffer .= $_; next;
+               if ($_chunk_size == 1) {
+                  $_buf = <$_input_glob>;
+                  $_eof_flag = 1 unless (length $_buf);
+               }
+               else {
+                  my $_last_len = 0;
+                  for (1 .. $_chunk_size) {
+                     $_buf .= <$_input_glob>;
+                     $_len  = length $_buf;
+                     if ($_len == $_last_len) {
+                        $_eof_flag = 1;
+                        last;
+                     }
+                     $_last_len = $_len;
                   }
-                  $_eof_flag = 1; last;
                }
             }
             else {
-               if (read($_input_glob, $_buffer, $_chunk_size) == $_chunk_size) {
-                  if (defined($_ = <$_input_glob>)) {
-                     $_buffer .= $_;
-                  } else {
-                     $_eof_flag = 1;
-                  }
+               if (read($_input_glob, $_buf, $_chunk_size) == $_chunk_size) {
+                  $_buf .= <$_input_glob>;
+                  $_eof_flag = 1 if (length $_buf == $_chunk_size);
+               }
+               else {
+                  $_eof_flag = 1;
                }
             }
          }
 
-         local $\ = undef if (defined $\); $_len = length $_buffer;
+         $_len = length $_buf; local $\ = undef if (defined $\);
 
-         print $_DAU_R_SOCK ($_len)
-            ? $_len . $LF . (++$_chunk_id) . $LF . $_buffer
-            : '0' . $LF;
+         if ($_len) {
+            if ($_len < FAST_SEND_SIZE) {
+               print {$_DAU_R_SOCK} $_len . $LF . (++$_chunk_id) . $LF . $_buf;
+            } else {
+               print {$_DAU_R_SOCK} $_len . $LF . (++$_chunk_id) . $LF;
+               print {$_DAU_R_SOCK} $_buf;
+            }
+         }
+         else {
+            print {$_DAU_R_SOCK} '0' . $LF;
+         }
+
+         return;
+      },
+
+      OUTPUT_U_ITR.$LF => sub {                   ## User << Iterator
+         my $_buf;
+
+         if ($_aborted) {
+            local $\ = undef if (defined $\);
+            print {$_DAU_R_SOCK} '-1' . $LF;
+            return;
+         }
+
+         my @_ret_a = $_input_data->($_chunk_size);
+
+         if (scalar @_ret_a > 1 || ref $_ret_a[0]) {
+            $_buf = $self->{freeze}( [ @_ret_a ] );
+            $_len = length $_buf; local $\ = undef if (defined $\);
+
+            if ($_len < FAST_SEND_SIZE) {
+               print {$_DAU_R_SOCK}
+                  $_len . '1' . $LF . (++$_chunk_id) . $LF . $_buf;
+            } else {
+               print {$_DAU_R_SOCK} $_len . '1' . $LF . (++$_chunk_id) . $LF;
+               print {$_DAU_R_SOCK} $_buf;
+            }
+
+            return;
+         }
+         elsif (defined $_ret_a[0]) {
+            $_len = length $_ret_a[0]; local $\ = undef if (defined $\);
+
+            if ($_len < FAST_SEND_SIZE) {
+               print {$_DAU_R_SOCK}
+                  $_len . '0' . $LF . (++$_chunk_id) . $LF . $_ret_a[0];
+            } else {
+               print {$_DAU_R_SOCK} $_len . '0' . $LF . (++$_chunk_id) . $LF;
+               print {$_DAU_R_SOCK} $_ret_a[0];
+            }
+
+            return;
+         }
+
+         local $\ = undef if (defined $\);
+         print {$_DAU_R_SOCK} '-1' . $LF;
+         $_aborted = 1;
 
          return;
       },
@@ -286,109 +421,77 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_A_CBK.$LF => sub {                   ## Callback w/ multiple args
-         my $_buffer;
+         my ($_buf, $_data_ref);
 
-         chomp($_want_id  = <$_DAU_R_SOCK>);
-         chomp($_callback = <$_DAU_R_SOCK>);
-         chomp($_len      = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len;
+         chomp($_wa  = <$_DAU_R_SOCK>);
+         chomp($_cb  = <$_DAU_R_SOCK>);
+         chomp($_len = <$_DAU_R_SOCK>);
+         read $_DAU_R_SOCK, $_buf, $_len;
 
-         my $_data_ref = $self->{thaw}($_buffer);
-         undef $_buffer;
+         $_data_ref = $self->{thaw}($_buf); undef $_buf;
 
          local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
          no strict 'refs';
 
-         if ($_want_id == WANTS_UNDEF) {
-            $_callback->(@{ $_data_ref });
+         if ($_wa == WANTS_UNDEF) {
+            $_cb->(@{ $_data_ref });
          }
-         elsif ($_want_id == WANTS_ARRAY) {
-            my @_ret_a = $_callback->(@{ $_data_ref });
-            $_buffer = $self->{freeze}(\@_ret_a);
-            local $\ = undef if (defined $\); $_len = length $_buffer;
-            print $_DAU_R_SOCK $_len . $LF . $_buffer;
+         elsif ($_wa == WANTS_ARRAY) {
+            my @_ret_a = $_cb->(@{ $_data_ref });
+            $_cb_ret_a->(\@_ret_a);
          }
          else {
-            my $_ret_s = $_callback->(@{ $_data_ref });
-            unless (ref $_ret_s) {
-               local $\ = undef if (defined $\); $_len = length $_ret_s || 0;
-               print $_DAU_R_SOCK WANTS_SCALAR . $LF . $_len . $LF . $_ret_s;
-            }
-            else {
-               $_buffer = $self->{freeze}($_ret_s);
-               local $\ = undef if (defined $\); $_len = length $_buffer;
-               print $_DAU_R_SOCK WANTS_REF . $LF . $_len . $LF . $_buffer;
-            }
+            my  $_ret_s = $_cb->(@{ $_data_ref });
+            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
          }
 
          return;
       },
 
       OUTPUT_S_CBK.$LF => sub {                   ## Callback w/ 1 scalar arg
-         my $_buffer;
+         my $_buf;
 
-         chomp($_want_id  = <$_DAU_R_SOCK>);
-         chomp($_callback = <$_DAU_R_SOCK>);
-         chomp($_len      = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len;
+         chomp($_wa  = <$_DAU_R_SOCK>);
+         chomp($_cb  = <$_DAU_R_SOCK>);
+         chomp($_len = <$_DAU_R_SOCK>);
+         read $_DAU_R_SOCK, $_buf, $_len;
 
          local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
          no strict 'refs';
 
-         if ($_want_id == WANTS_UNDEF) {
-            $_callback->($_buffer);
+         if ($_wa == WANTS_UNDEF) {
+            $_cb->($_buf);
          }
-         elsif ($_want_id == WANTS_ARRAY) {
-            my @_ret_a = $_callback->($_buffer);
-            $_buffer = $self->{freeze}(\@_ret_a);
-            local $\ = undef if (defined $\); $_len = length $_buffer;
-            print $_DAU_R_SOCK $_len . $LF . $_buffer;
+         elsif ($_wa == WANTS_ARRAY) {
+            my @_ret_a = $_cb->($_buf);
+            $_cb_ret_a->(\@_ret_a);
          }
          else {
-            my $_ret_s = $_callback->($_buffer);
-            unless (ref $_ret_s) {
-               local $\ = undef if (defined $\); $_len = length $_ret_s || 0;
-               print $_DAU_R_SOCK WANTS_SCALAR . $LF . $_len . $LF . $_ret_s;
-            }
-            else {
-               $_buffer = $self->{freeze}($_ret_s);
-               local $\ = undef if (defined $\); $_len = length $_buffer;
-               print $_DAU_R_SOCK WANTS_REF . $LF . $_len . $LF . $_buffer;
-            }
+            my  $_ret_s = $_cb->($_buf);
+            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
          }
 
          return;
       },
 
       OUTPUT_N_CBK.$LF => sub {                   ## Callback w/ no args
-         my $_buffer;
 
-         chomp($_want_id  = <$_DAU_R_SOCK>);
-         chomp($_callback = <$_DAU_R_SOCK>);
+         chomp($_wa = <$_DAU_R_SOCK>);
+         chomp($_cb = <$_DAU_R_SOCK>);
 
          local $\ = $_O_SEP if ($_O_FLG); local $/ = $_I_SEP if ($_I_FLG);
          no strict 'refs';
 
-         if ($_want_id == WANTS_UNDEF) {
-            $_callback->();
+         if ($_wa == WANTS_UNDEF) {
+            $_cb->();
          }
-         elsif ($_want_id == WANTS_ARRAY) {
-            my @_ret_a = $_callback->();
-            $_buffer = $self->{freeze}(\@_ret_a);
-            local $\ = undef if (defined $\); $_len = length $_buffer;
-            print $_DAU_R_SOCK $_len . $LF . $_buffer;
+         elsif ($_wa == WANTS_ARRAY) {
+            my @_ret_a = $_cb->();
+            $_cb_ret_a->(\@_ret_a);
          }
          else {
-            my $_ret_s = $_callback->();
-            unless (ref $_ret_s) {
-               local $\ = undef if (defined $\); $_len = length $_ret_s || 0;
-               print $_DAU_R_SOCK WANTS_SCALAR . $LF . $_len . $LF . $_ret_s;
-            }
-            else {
-               $_buffer = $self->{freeze}($_ret_s);
-               local $\ = undef if (defined $\); $_len = length $_buffer;
-               print $_DAU_R_SOCK WANTS_REF . $LF . $_len . $LF . $_buffer;
-            }
+            my  $_ret_s = $_cb->();
+            ref $_ret_s ? $_cb_ret_r->($_ret_s) : $_cb_ret_s->($_ret_s);
          }
 
          return;
@@ -397,54 +500,77 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_A_GTR.$LF => sub {                   ## Gather w/ multiple args
-         my $_buffer;
+         my $_buf;
 
          chomp($_task_id = <$_DAU_R_SOCK>);
          chomp($_len     = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len;
+         read $_DAU_R_SOCK, $_buf, $_len;
 
          if ($_is_c_ref[$_task_id]) {
-            local $_ = $self->{thaw}($_buffer); $_gather[$_task_id]->(@{ $_ });
-         } elsif ($_is_q_ref[$_task_id]) {
-            $_gather[$_task_id]->enqueue( @{ $self->{thaw}($_buffer) } );
-         } else {
-            push @{ $_gather[$_task_id] }, @{ $self->{thaw}($_buffer) };
+            local $_ = $self->{thaw}($_buf);
+            $_gather[$_task_id]->(@{ $_ });
+         }
+         elsif ($_is_h_ref[$_task_id]) {
+            local $_ = $self->{thaw}($_buf);
+            while (1) {
+               my $_key = shift @{ $_ }; my $_val = shift @{ $_ };
+               $_gather[$_task_id]->{$_key} = $_val;
+               last unless (@{ $_ });
+            }
+         }
+         elsif ($_is_q_ref[$_task_id]) {
+            $_gather[$_task_id]->enqueue(@{ $self->{thaw}($_buf) });
+         }
+         else {
+            push @{ $_gather[$_task_id] }, @{ $self->{thaw}($_buf) };
          }
 
          return;
       },
 
       OUTPUT_R_GTR.$LF => sub {                   ## Gather w/ 1 reference arg
-         my $_buffer;
+         my $_buf;
 
          chomp($_task_id = <$_DAU_R_SOCK>);
          chomp($_len     = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len;
+         read $_DAU_R_SOCK, $_buf, $_len;
 
          if ($_is_c_ref[$_task_id]) {
-            local $_ = $self->{thaw}($_buffer); $_gather[$_task_id]->($_);
-         } elsif ($_is_q_ref[$_task_id]) {
-            $_gather[$_task_id]->enqueue( $self->{thaw}($_buffer) );
-         } else {
-            push @{ $_gather[$_task_id] }, $self->{thaw}($_buffer);
+            local $_ = $self->{thaw}($_buf);
+            $_gather[$_task_id]->($_);
+         }
+         elsif ($_is_h_ref[$_task_id]) {
+            local $_ = $self->{thaw}($_buf);
+            $_gather[$_task_id]->{$_} = undef;
+         }
+         elsif ($_is_q_ref[$_task_id]) {
+            $_gather[$_task_id]->enqueue($self->{thaw}($_buf));
+         }
+         else {
+            push @{ $_gather[$_task_id] }, $self->{thaw}($_buf);
          }
 
          return;
       },
 
       OUTPUT_S_GTR.$LF => sub {                   ## Gather w/ 1 scalar arg
-         my $_buffer;
+         local $_;
 
          chomp($_task_id = <$_DAU_R_SOCK>);
          chomp($_len     = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len if ($_len >= 0);
+         read $_DAU_R_SOCK, $_, $_len if ($_len >= 0);
 
          if ($_is_c_ref[$_task_id]) {
-            local $_ = $_buffer; $_gather[$_task_id]->($_);
-         } elsif ($_is_q_ref[$_task_id]) {
-            $_gather[$_task_id]->enqueue( $_buffer );
-         } else {
-            push @{ $_gather[$_task_id] }, $_buffer;
+            $_gather[$_task_id]->($_);
+         }
+         elsif ($_is_h_ref[$_task_id]) {
+            $_gather[$_task_id]->{$_} = undef;
+         }
+         elsif ($_is_q_ref[$_task_id]) {
+            $_gather[$_task_id]->enqueue($_);
+         }
+         else {
+            push @{ $_gather[$_task_id] }, $_;
          }
 
          return;
@@ -453,41 +579,41 @@ sub _output_loop {
       ## ----------------------------------------------------------------------
 
       OUTPUT_O_SND.$LF => sub {                   ## Send >> STDOUT
-         my $_buffer;
+         my $_buf;
 
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len;
+         read $_DAU_R_SOCK, $_buf, $_len;
 
          if (defined $_user_output) {
-            $_user_output->($_buffer);
+            $_user_output->($_buf);
          } else {
-            print $_MCE_STDOUT $_buffer;
+            print {$_MCE_STDOUT} $_buf;
          }
 
          return;
       },
 
       OUTPUT_E_SND.$LF => sub {                   ## Send >> STDERR
-         my $_buffer;
+         my $_buf;
 
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len;
+         read $_DAU_R_SOCK, $_buf, $_len;
 
          if (defined $_user_error) {
-            $_user_error->($_buffer);
+            $_user_error->($_buf);
          } else {
-            print $_MCE_STDERR $_buffer;
+            print {$_MCE_STDERR} $_buf;
          }
 
          return;
       },
 
       OUTPUT_F_SND.$LF => sub {                   ## Send >> File
-         my ($_buffer, $_OUT_FILE);
+         my ($_buf, $_OUT_FILE);
 
          chomp($_file = <$_DAU_R_SOCK>);
          chomp($_len  = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len;
+         read $_DAU_R_SOCK, $_buf, $_len;
 
          unless (exists $_sendto_fhs{$_file}) {
             open $_sendto_fhs{$_file}, '>>', $_file
@@ -496,22 +622,23 @@ sub _output_loop {
             binmode $_sendto_fhs{$_file};
 
             ## Select new FH, turn on autoflush, restore the old FH.
-            select((select($_sendto_fhs{$_file}), $| = 1)[0])
-               if ($_flush_file);
+            if ($_flush_file) {
+               local $|; select((select($_sendto_fhs{$_file}), $| = 1)[0]);
+            }
          }
 
          $_OUT_FILE = $_sendto_fhs{$_file};
-         print $_OUT_FILE $_buffer;
+         print {$_OUT_FILE} $_buf;
 
          return;
       },
 
       OUTPUT_D_SND.$LF => sub {                   ## Send >> File descriptor
-         my ($_buffer, $_OUT_FILE);
+         my ($_buf, $_OUT_FILE);
 
          chomp($_fd  = <$_DAU_R_SOCK>);
          chomp($_len = <$_DAU_R_SOCK>);
-         read $_DAU_R_SOCK, $_buffer, $_len;
+         read $_DAU_R_SOCK, $_buf, $_len;
 
          unless (exists $_sendto_fhs{$_fd}) {
             require IO::Handle unless (defined $IO::Handle::VERSION);
@@ -523,12 +650,13 @@ sub _output_loop {
             binmode $_sendto_fhs{$_fd};
 
             ## Select new FH, turn on autoflush, restore the old FH.
-            select((select($_sendto_fhs{$_fd}), $| = 1)[0])
-               if ($_flush_file);
+            if ($_flush_file) {
+               local $|; select((select($_sendto_fhs{$_fd}), $| = 1)[0]);
+            }
          }
 
          $_OUT_FILE = $_sendto_fhs{$_fd};
-         print $_OUT_FILE $_buffer;
+         print {$_OUT_FILE} $_buf;
 
          return;
       },
@@ -565,51 +693,72 @@ sub _output_loop {
          }
 
          return;
-      }
+      },
 
    );
 
    ## -------------------------------------------------------------------------
 
-   $_has_user_tasks = (defined $self->{user_tasks});
+   local ($!, $_);
+
+   $_has_user_tasks = (defined $self->{user_tasks}) ? 1 : 0;
+   $_cs_one_flag = ($self->{chunk_size} == 1) ? 1 : 0;
    $_aborted = $_chunk_id = $_eof_flag = 0;
 
+   $_max_retries  = $self->{max_retries};
    $_on_post_exit = $self->{on_post_exit};
    $_on_post_run  = $self->{on_post_run};
    $_chunk_size   = $self->{chunk_size};
    $_flush_file   = $self->{flush_file};
-   $_use_slurpio  = $self->{use_slurpio};
    $_user_output  = $self->{user_output};
    $_user_error   = $self->{user_error};
    $_single_dim   = $self->{_single_dim};
    $_sess_dir     = $self->{_sess_dir};
 
-   $_chunk_size = $self->{user_tasks}->[0]->{chunk_size}
-      if ($_has_user_tasks && $self->{user_tasks}->[0]->{chunk_size});
+   if ($_max_retries && !$_on_post_exit) {
+      $_on_post_exit = sub {
+         my ($self, $_e, $_retry_cnt) = @_;
+         my ($_cnt, $_msg) = ($_retry_cnt + 1, "Error: Chunk $_e->{id} failed");
+
+         ($_retry_cnt < $_max_retries)
+            ? print {*STDERR} "$_msg, retrying chunk attempt #${_cnt}\n"
+            : print {*STDERR} "$_msg\n";
+
+         $self->restart_worker;
+      };
+   }
+
+   if ($_has_user_tasks && $self->{user_tasks}->[0]->{chunk_size}) {
+      $_chunk_size = $self->{user_tasks}->[0]->{chunk_size};
+   }
 
    if ($_has_user_tasks) {
-      foreach (0 .. @{ $self->{user_tasks} } - 1) {
-         $_gather[$_] = (defined $self->{user_tasks}->[$_]->{gather})
-            ? $self->{user_tasks}->[$_]->{gather} : $self->{gather};
-         $_is_c_ref[$_] = (
-            ref $_gather[$_] eq 'CODE' ) ? 1 : 0;
-         $_is_q_ref[$_] = (
-            ref $_gather[$_] eq 'MCE::Queue' ||
-            ref $_gather[$_] eq 'Thread::Queue' ) ? 1 : 0;
+      for my $_i (0 .. @{ $self->{user_tasks} } - 1) {
+         $_gather[$_i] = (defined $self->{user_tasks}->[$_i]->{gather})
+            ? $self->{user_tasks}->[$_i]->{gather} : $self->{gather};
+
+         $_is_c_ref[$_i] = ( ref $_gather[$_i] eq 'CODE' ) ? 1 : 0;
+         $_is_h_ref[$_i] = ( ref $_gather[$_i] eq 'HASH' ) ? 1 : 0;
+
+         $_is_q_ref[$_i] = (
+            ref $_gather[$_i] eq 'MCE::Queue' ||
+            ref $_gather[$_i] eq 'Thread::Queue' ) ? 1 : 0;
       }
    }
 
    if (defined $self->{gather}) {
       $_gather[0] = $self->{gather};
-      $_is_c_ref[0] = (
-         ref $_gather[0] eq 'CODE' ) ? 1 : 0;
+
+      $_is_c_ref[0] = ( ref $_gather[0] eq 'CODE' ) ? 1 : 0;
+      $_is_h_ref[0] = ( ref $_gather[0] eq 'HASH' ) ? 1 : 0;
+
       $_is_q_ref[0] = (
          ref $_gather[0] eq 'MCE::Queue' ||
          ref $_gather[0] eq 'Thread::Queue' ) ? 1 : 0;
    }
 
    if (defined $_input_data && ref $_input_data eq 'ARRAY') {
-      $_input_size = @$_input_data;
+      $_input_size = @{ $_input_data };
       $_offset_pos = 0;
    } else {
       $_input_size = $_offset_pos = 0;
@@ -622,7 +771,7 @@ sub _output_loop {
       binmode $_MCE_STDOUT;
    }
    else {
-      open $_MCE_STDOUT, ">&=STDOUT";
+      open $_MCE_STDOUT, '>&=STDOUT';
       binmode $_MCE_STDOUT;
    }
 
@@ -632,65 +781,71 @@ sub _output_loop {
       binmode $_MCE_STDERR;
    }
    else {
-      open $_MCE_STDERR, ">&=STDERR";
+      open $_MCE_STDERR, '>&=STDERR';
       binmode $_MCE_STDERR;
    }
 
    ## Make MCE_STDOUT the default handle.
    ## Flush STDERR/STDOUT handles if requested.
    my $_old_hndl = select $_MCE_STDOUT;
-   $| = 1 if ($self->{flush_stdout});
 
+   if ($self->{flush_stdout}) {
+      local $|; select((select($_MCE_STDOUT), $| = 1)[0]);
+   }
    if ($self->{flush_stderr}) {
-      select $_MCE_STDERR; $| = 1;
-      select $_MCE_STDOUT;
+      local $|; select((select($_MCE_STDERR), $| = 1)[0]);
    }
 
    ## -------------------------------------------------------------------------
    ## Output event loop.
 
-   my ($_func, $_chn); my $_channels = $self->{_dat_r_sock};
+   my $_func; my $_channels = $self->{_dat_r_sock};
 
    $_BSB_W_SOCK = $self->{_bsb_w_sock};        ## For IPC
    $_BSE_W_SOCK = $self->{_bse_w_sock};
    $_DAT_R_SOCK = $self->{_dat_r_sock}->[0];
 
-   $_RS    = $self->{RS} || $/;    $_RS_FLG = (!$_RS || $_RS ne $LF);
-   $_O_SEP = $\; local $\ = undef; $_O_FLG  = defined $_O_SEP;
-   $_I_SEP = $/; local $/ = $LF;   $_I_FLG  = (!$_I_SEP || $_I_SEP ne $LF);
+   $_RS     = $self->{RS} || $/;
+   $_O_SEP  = $\; local $\ = undef;
+   $_I_SEP  = $/; local $/ = $LF;
+
+   $_RS_FLG = (!$_RS || $_RS ne $LF) ? 1 : 0;
+   $_O_FLG  = (defined $_O_SEP) ? 1 : 0;
+   $_I_FLG  = (!$_I_SEP || $_I_SEP ne $LF) ? 1 : 0;
 
    ## Call module's loop_begin routine for modules plugged into MCE.
-   $_->($self, \$_DAU_R_SOCK) for (@{ $_plugin_loop_begin });
+   for my $_p (@{ $_plugin_loop_begin }) {
+      $_p->($self, \$_DAU_R_SOCK);
+   }
 
    ## Call on hash function. Exit loop when workers have completed.
    while (1) {
       $_func = <$_DAT_R_SOCK>;
-      next unless (defined $_func);
+      next unless $_func;
 
       $_DAU_R_SOCK = $_channels->[ <$_DAT_R_SOCK> ];
-
       if (exists $_core_output_function{$_func}) {
          $_core_output_function{$_func}();
-      }
-      elsif (exists $_plugin_function->{$_func}) {
+      } elsif (exists $_plugin_function->{$_func}) {
          $_plugin_function->{$_func}();
       }
 
-      last unless ($self->{_total_running});
+      last unless $self->{_total_running};
    }
 
    ## Call module's loop_end routine for modules plugged into MCE.
-   $_->($self) for (@{ $_plugin_loop_end });
+   for my $_p (@{ $_plugin_loop_end }) {
+      $_p->($self);
+   }
 
    ## Call on_post_run callback.
-   $_on_post_run->($self, $self->{_status})
-      if (defined $_on_post_run);
+   $_on_post_run->($self, $self->{_status}) if (defined $_on_post_run);
 
    ## Close opened sendto file handles.
-   for (keys %_sendto_fhs) {
-      close  $_sendto_fhs{$_};
-      undef  $_sendto_fhs{$_};
-      delete $_sendto_fhs{$_};
+   for my $_p (keys %_sendto_fhs) {
+      close  $_sendto_fhs{$_p};
+      undef  $_sendto_fhs{$_p};
+      delete $_sendto_fhs{$_p};
    }
 
    ## Restore the default handle. Close MCE STDOUT/STDERR handles.
