@@ -9,12 +9,19 @@ package MCE::Loop;
 use strict;
 use warnings;
 
+no warnings qw( threads recursion uninitialized );
+
+our $VERSION = '1.698_001';
+
+## no critic (BuiltinFunctions::ProhibitStringyEval)
+## no critic (Subroutines::ProhibitSubroutinePrototypes)
+## no critic (TestingAndDebugging::ProhibitNoStrict)
+
 use Scalar::Util qw( looks_like_number );
+use Storable ();
+use MCE::Signal;
 
-use MCE;
-use MCE::Util;
-
-our $VERSION = '1.499_005'; $VERSION = eval $VERSION;
+our @CARP_NOT = qw( MCE );
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -22,38 +29,43 @@ our $VERSION = '1.499_005'; $VERSION = eval $VERSION;
 ##
 ###############################################################################
 
-our $MAX_WORKERS = 'auto';
-our $CHUNK_SIZE  = 'auto';
+my ($MAX_WORKERS, $CHUNK_SIZE) = ('auto', 'auto');
+
+my $TMP_DIR = $MCE::Signal::tmp_dir;
+my $FREEZE  = \&Storable::freeze;
+my $THAW    = \&Storable::thaw;
 
 my ($_MCE, $_loaded); my ($_params, $_prev_c); my $_tag = 'MCE::Loop';
 
 sub import {
-
    my $_class = shift; return if ($_loaded++);
 
    ## Process module arguments.
-   while (my $_arg = shift) {
+   while (my $_argument = shift) {
+      my $_arg = lc $_argument;
 
-      $MAX_WORKERS  = shift and next if ( $_arg =~ /^max_workers$/i );
-      $CHUNK_SIZE   = shift and next if ( $_arg =~ /^chunk_size$/i );
-      $MCE::TMP_DIR = shift and next if ( $_arg =~ /^tmp_dir$/i );
-      $MCE::FREEZE  = shift and next if ( $_arg =~ /^freeze$/i );
-      $MCE::THAW    = shift and next if ( $_arg =~ /^thaw$/i );
+      $MAX_WORKERS = shift, next if ( $_arg eq 'max_workers' );
+      $CHUNK_SIZE  = shift, next if ( $_arg eq 'chunk_size' );
+      $FREEZE      = shift, next if ( $_arg eq 'freeze' );
+      $THAW        = shift, next if ( $_arg eq 'thaw' );
+      $TMP_DIR     = shift, next if ( $_arg eq 'tmp_dir' );
 
-      if ( $_arg =~ /^sereal$/i ) {
+      if ( $_arg eq 'sereal' ) {
          if (shift eq '1') {
             local $@; eval 'use Sereal qw(encode_sereal decode_sereal)';
-            unless ($@) {
-               $MCE::FREEZE = \&encode_sereal;
-               $MCE::THAW   = \&decode_sereal;
-            }
+            $FREEZE = \&encode_sereal, $THAW = \&decode_sereal unless $@;
          }
          next;
       }
 
-      _croak("$_tag::import: '$_arg' is not a valid module argument");
+      _croak("Error: ($_argument) invalid module option");
    }
 
+   ## Preload essential modules.
+   require MCE; MCE->import(
+      freeze => $FREEZE, thaw => $THAW, tmp_dir => $TMP_DIR
+   );
+
    $MAX_WORKERS = MCE::Util::_parse_max_workers($MAX_WORKERS);
    _validate_number($MAX_WORKERS, 'MAX_WORKERS');
 
@@ -62,17 +74,19 @@ sub import {
 
    ## Import functions.
    no strict 'refs'; no warnings 'redefine';
-   my $_package = caller();
+   my $_pkg = caller;
 
-   *{ $_package . '::mce_loop_f' } = \&mce_loop_f;
-   *{ $_package . '::mce_loop_s' } = \&mce_loop_s;
-   *{ $_package . '::mce_loop'   } = \&mce_loop;
+   *{ $_pkg.'::mce_loop_f' } = \&run_file;
+   *{ $_pkg.'::mce_loop_s' } = \&run_seq;
+   *{ $_pkg.'::mce_loop'   } = \&run;
 
    return;
 }
 
 END {
-   MCE::Loop::finish();
+   return if (defined $_MCE && $_MCE->wid);
+
+   finish();
 }
 
 ###############################################################################
@@ -83,24 +97,23 @@ END {
 
 sub init (@) {
 
+   shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
+
    if (MCE->wid) {
-      @_ = (); _croak(
-         "$_tag: function cannot be called by the worker process"
-      );
+      @_ = (); _croak("$_tag: (init) is not allowed by worker process");
    }
 
-   _croak("$_tag: 'argument' is not a HASH reference")
-      unless (ref $_[0] eq 'HASH');
+   finish(); $_params = (ref $_[0] eq 'HASH') ? shift : { @_ };
 
-   MCE::Loop::finish(); $_params = shift;
+   @_ = ();
 
    return;
 }
 
 sub finish () {
 
-   if (defined $_MCE) {
-      MCE::_save_state; $_MCE->shutdown(); MCE::_restore_state;
+   if (defined $_MCE && $_MCE->{_spawned}) {
+      MCE::_save_state(); $_MCE->shutdown(); MCE::_restore_state();
    }
 
    $_prev_c = undef;
@@ -114,7 +127,9 @@ sub finish () {
 ##
 ###############################################################################
 
-sub mce_loop_f (&@) {
+sub run_file (&@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
 
    my $_code = shift; my $_file = shift;
 
@@ -126,22 +141,22 @@ sub mce_loop_f (&@) {
       $_params = {};
    }
 
-   if (defined $_file && ref $_file eq "" && $_file ne "") {
-      _croak("$_tag: '$_file' does not exist") unless (-e $_file);
-      _croak("$_tag: '$_file' is not readable") unless (-r $_file);
-      _croak("$_tag: '$_file' is not a plain file") unless (-f $_file);
+   if (defined $_file && ref $_file eq '' && $_file ne '') {
+      _croak("$_tag: ($_file) does not exist") unless (-e $_file);
+      _croak("$_tag: ($_file) is not readable") unless (-r $_file);
+      _croak("$_tag: ($_file) is not a plain file") unless (-f $_file);
       $_params->{_file} = $_file;
    }
-   elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR') {
+   elsif (ref $_file eq 'GLOB' || ref $_file eq 'SCALAR' || ref($_file) =~ /^IO::/) {
       $_params->{_file} = $_file;
    }
    else {
-      _croak("$_tag: 'file' is not specified or valid");
+      _croak("$_tag: (file) is not specified or valid");
    }
 
    @_ = ();
 
-   return mce_loop($_code);
+   return run($_code);
 }
 
 ###############################################################################
@@ -150,7 +165,9 @@ sub mce_loop_f (&@) {
 ##
 ###############################################################################
 
-sub mce_loop_s (&@) {
+sub run_seq (&@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
 
    my $_code = shift;
 
@@ -172,23 +189,25 @@ sub mce_loop_s (&@) {
       $_begin = $_[0]->[0]; $_end = $_[0]->[1];
       $_params->{sequence} = $_[0];
    }
-   elsif (ref $_[0] eq "") {
+   elsif (ref $_[0] eq '') {
       $_begin = $_[0]; $_end = $_[1];
       $_params->{sequence} = [ @_ ];
    }
    else {
-      _croak("$_tag: 'sequence' is not specified or valid");
+      _croak("$_tag: (sequence) is not specified or valid");
    }
 
-   _croak("$_tag: 'begin' is not specified for sequence")
+   _croak("$_tag: (begin) is not specified for sequence")
       unless (defined $_begin);
 
-   _croak("$_tag: 'end' is not specified for sequence")
+   _croak("$_tag: (end) is not specified for sequence")
       unless (defined $_end);
 
+   $_params->{sequence_run} = 1;
+
    @_ = ();
 
-   return mce_loop($_code);
+   return run($_code);
 }
 
 ###############################################################################
@@ -197,19 +216,19 @@ sub mce_loop_s (&@) {
 ##
 ###############################################################################
 
-sub mce_loop (&@) {
+sub run (&@) {
+
+   shift if (defined $_[0] && $_[0] eq 'MCE::Loop');
 
    my $_code = shift;
 
    if (MCE->wid) {
-      @_ = (); _croak(
-         "$_tag: function cannot be called by the worker process"
-      );
+      @_ = (); _croak("$_tag: (run) is not allowed by worker process");
    }
 
    my $_input_data; my $_max_workers = $MAX_WORKERS; my $_r = ref $_[0];
 
-   if ($_r eq 'ARRAY' || $_r eq 'GLOB' || $_r eq 'SCALAR') {
+   if ($_r eq 'ARRAY' || $_r eq 'CODE' || $_r eq 'GLOB' || $_r eq 'SCALAR' || $_r =~ /^IO::/) {
       $_input_data = shift;
    }
 
@@ -217,6 +236,7 @@ sub mce_loop (&@) {
       $_max_workers = MCE::Util::_parse_max_workers($_p->{max_workers})
          if (exists $_p->{max_workers});
 
+      delete $_p->{sequence}   if (defined $_input_data || scalar @_);
       delete $_p->{user_func}  if (exists $_p->{user_func});
       delete $_p->{user_tasks} if (exists $_p->{user_tasks});
    }
@@ -226,52 +246,81 @@ sub mce_loop (&@) {
    );
 
    if (defined $_params) {
-      $_input_data = $_params->{input_data} if (exists $_params->{input_data});
-      $_input_data = $_params->{_file} if (exists $_params->{_file});
+      if (exists $_params->{_file}) {
+         $_input_data = delete $_params->{_file};
+      }
+      else {
+         $_input_data = $_params->{input_data} if exists $_params->{input_data};
+      }
    }
 
-   ## -------------------------------------------------------------------------
+   MCE::_save_state();
 
-   MCE::_save_state;
+   ## -------------------------------------------------------------------------
 
    if (!defined $_prev_c || $_prev_c != $_code) {
       $_MCE->shutdown() if (defined $_MCE);
       $_prev_c = $_code;
 
-      $_MCE = MCE->new(
+      my %_options = (
          max_workers => $_max_workers, task_name => $_tag,
-         user_func => $_code
+         user_func => $_code,
       );
 
       if (defined $_params) {
-         my $_p = $_params; foreach (keys %{ $_p }) {
-            next if ($_ eq 'input_data');
-            $_MCE->{$_} = $_p->{$_};
+         for my $_p (keys %{ $_params }) {
+            next if ($_p eq 'sequence_run');
+            next if ($_p eq 'input_data');
+            next if ($_p eq 'chunk_size');
+
+            _croak("MCE::Loop: ($_p) is not a valid constructor argument")
+               unless (exists $MCE::_valid_fields_new{$_p});
+
+            $_options{$_p} = $_params->{$_p};
          }
       }
+
+      $_MCE = MCE->new(%_options);
    }
 
+   ## -------------------------------------------------------------------------
+
    my @_a; my $_wa = wantarray; $_MCE->{gather} = \@_a if (defined $_wa);
 
    if (defined $_input_data) {
-      @_ = (); $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
+      @_ = ();
+      $_MCE->process({ chunk_size => $_chunk_size }, $_input_data);
+      delete $_MCE->{input_data};
    }
    elsif (scalar @_) {
       $_MCE->process({ chunk_size => $_chunk_size }, \@_);
+      delete $_MCE->{input_data};
    }
    else {
-      $_MCE->run({ chunk_size => $_chunk_size }, 0)
-         if (defined $_params && exists $_params->{sequence});
-   }
-
-   if (defined $_params) {
-      delete $_params->{input_data}; delete $_params->{_file};
-      delete $_params->{sequence};
+      if (defined $_params && exists $_params->{sequence}) {
+         $_MCE->run({
+            chunk_size => $_chunk_size, sequence => $_params->{sequence}
+         }, 0);
+         if (exists $_params->{sequence_run}) {
+            delete $_params->{sequence_run};
+            delete $_params->{sequence};
+         }
+         delete $_MCE->{sequence};
+      }
    }
 
    delete $_MCE->{gather} if (defined $_wa);
 
-   MCE::_restore_state;
+   MCE::_restore_state();
+
+   if (exists $_MCE->{_rla_return}) {
+      $MCE::MCE->{_rla_return} = delete $_MCE->{_rla_return};
+   }
+
+   if ($^S) {
+      ## shutdown if in eval state
+      MCE::_save_state(); $_MCE->shutdown(); MCE::_restore_state();
+   }
 
    return ((defined $_wa) ? @_a : ());
 }
@@ -289,10 +338,15 @@ sub _croak {
 
 sub _validate_number {
 
-   my $_n = $_[0]; my $_key = $_[1];
+   my ($_n, $_key) = @_;
 
-   _croak("$_tag: '$_key' is not valid")
-      if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1);
+   _croak("$_tag: ($_key) is not valid") if (!defined $_n);
+
+   $_n =~ s/K\z//i; $_n =~ s/M\z//i;
+
+   if (!looks_like_number($_n) || int($_n) != $_n || $_n < 1) {
+      _croak("$_tag: ($_key) is not valid");
+   }
 
    return;
 }
@@ -313,25 +367,76 @@ MCE::Loop - Parallel loop model for building creative loops
 
 =head1 VERSION
 
-This document describes MCE::Loop version 1.499_005
+This document describes MCE::Loop version 1.698_001
 
 =head1 DESCRIPTION
 
-This module provides a parallel loop implementation through Many-core Engine.
-MCE::Loop is not MCE::Map but more along the lines of an easy way to spun up a
+This module provides a parallel loop implementation through Many-Core Engine.
+MCE::Loop is not MCE::Map but more along the lines of an easy way to spin up a
 MCE instance and have user_func pointing to your code block. If you want
-something similar to how map works, then also see L<MCE::Map>.
+something similar to map, then see L<MCE::Map|MCE::Map>.
+
+   ## Construction when chunking is not desired
+
+   use MCE::Loop;
+
+   MCE::Loop::init {
+      max_workers => 5, chunk_size => 1
+   };
+
+   mce_loop {
+      my ($mce, $chunk_ref, $chunk_id) = @_;
+      MCE->say("$chunk_id: $_");
+   } 40 .. 48;
+
+   -- Output
+
+   3: 42
+   1: 40
+   2: 41
+   4: 43
+   5: 44
+   6: 45
+   7: 46
+   8: 47
+   9: 48
+
+   ## Construction for 'auto' or greater than 1
+
+   use MCE::Loop;
+
+   MCE::Loop::init {
+      max_workers => 5, chunk_size => 'auto'
+   };
+
+   mce_loop {
+      my ($mce, $chunk_ref, $chunk_id) = @_;
+      for (@{ $chunk_ref }) {
+         MCE->say("$chunk_id: $_");
+      }
+   } 40 .. 48;
+
+   -- Output
+
+   1: 40
+   2: 42
+   1: 41
+   4: 46
+   2: 43
+   5: 48
+   3: 44
+   4: 47
+   3: 45
 
 =head1 SYNOPSIS when CHUNK_SIZE EQUALS 1
 
 All models in MCE default to 'auto' for chunk_size. The arguments for the block
-are the same as writing a user_func block for the core API.
+are the same as writing a user_func block using the Core API.
 
 Beginning with MCE 1.5, the next input item is placed into the input scalar
 variable $_ when chunk_size equals 1. Otherwise, $_ points to $chunk_ref
-containing many items. Basically, line 2 can be omitted from your code when
-using $_. One can call MCE->chunk_id to obtain the chunk_id for the current
-chunk.
+containing many items. Basically, line 2 below may be omitted from your code
+when using $_. One can call MCE->chunk_id to obtain the current chunk id.
 
    line 1:  user_func => sub {
    line 2:     my ($mce, $chunk_ref, $chunk_id) = @_;
@@ -344,7 +449,7 @@ chunk.
    line 9:  }
 
 Follow this synopsis when chunk_size equals one. Looping is not required from
-within the block. The block is called once per each item.
+inside the block. Hence, the block is called once per each item.
 
    ## Exports mce_loop, mce_loop_f, and mce_loop_s
    use MCE::Loop;
@@ -372,8 +477,8 @@ within the block. The block is called once per each item.
 
 =head1 SYNOPSIS when CHUNK_SIZE is GREATER THAN 1
 
-Follow this synopsis when chunk_size equals 'auto' or is greater than 1.
-This means having to loop through the chunk from within the block.
+Follow this synopsis when chunk_size equals 'auto' or greater than 1.
+This means having to loop through the chunk from inside the block.
 
    use MCE::Loop;
 
@@ -387,7 +492,7 @@ This means having to loop through the chunk from within the block.
 
    mce_loop { do_work($_) for (@{ $_ }) } 1..10000;
 
-   ## Same as above, resembles code using the core API.
+   ## Same as above, resembles code using the Core API.
 
    mce_loop {
       my ($mce, $chunk_ref, $chunk_id) = @_;
@@ -400,25 +505,26 @@ This means having to loop through the chunk from within the block.
 
 Chunking reduces the number of IPC calls behind the scene. Think in terms of
 chunks whenever processing a large amount of data. For relatively small data,
-choosing 1 for chunk_size is quite fine.
+choosing 1 for chunk_size is fine.
 
 =head1 OVERRIDING DEFAULTS
 
-The following list 5 options which may be overridden when loading the module.
+The following list options which may be overridden when loading the module.
 
-   use Sereal qw(encode_sereal decode_sereal);
+   use Sereal qw( encode_sereal decode_sereal );
+   use CBOR::XS qw( encode_cbor decode_cbor );
+   use JSON::XS qw( encode_json decode_json );
 
    use MCE::Loop
-         max_workers => 4,                    ## Default 'auto'
-         chunk_size  => 100,                  ## Default 'auto'
-         tmp_dir     => "/path/to/app/tmp",   ## $MCE::Signal::tmp_dir
-         freeze      => \&encode_sereal,      ## \&Storable::freeze
-         thaw        => \&decode_sereal       ## \&Storable::thaw
+         max_workers => 4,                ## Default 'auto'
+         chunk_size => 100,               ## Default 'auto'
+         tmp_dir => "/path/to/app/tmp",   ## $MCE::Signal::tmp_dir
+         freeze => \&encode_sereal,       ## \&Storable::freeze
+         thaw => \&decode_sereal          ## \&Storable::thaw
    ;
 
-There is a simplier way to enable Sereal with MCE 1.5. The following will
-attempt to use Sereal if available, otherwise will default back to using
-Storable for serialization.
+There is a simpler way to enable Sereal. The following will attempt to use
+Sereal if available, otherwise defaults to Storable for serialization.
 
    use MCE::Loop Sereal => 1;
 
@@ -426,16 +532,18 @@ Storable for serialization.
       chunk_size => 1
    };
 
-   ## Serialization is through Sereal if available.
+   ## Serialization is by the Sereal module if available.
    my %answer = mce_loop { MCE->gather( $_, sqrt $_ ) } 1..10000;
 
 =head1 CUSTOMIZING MCE
 
-=over 2
+=over 3
 
-=item init
+=item MCE::Loop->init ( options )
 
-The init function takes a hash of MCE options.
+=item MCE::Loop::init { options }
+
+The init function accepts a hash of MCE options.
 
    use MCE::Loop;
 
@@ -455,7 +563,7 @@ The init function takes a hash of MCE options.
 
    print "\n", "@a{1..100}", "\n";
 
-   -- output
+   -- Output
 
    ## 3 started
    ## 1 started
@@ -480,17 +588,30 @@ The init function takes a hash of MCE options.
 =head1 API DOCUMENTATION
 
 The following assumes chunk_size equals 1 in order to demonstrate all the
-possiblities of passing input data for the loop.
+possibilities of passing input data into the code block.
 
-=over 2
+=over 3
+
+=item MCE::Loop->run ( sub { code }, iterator )
+
+=item mce_loop { code } iterator
+
+An iterator reference can by specified for input_data. Iterators are described
+under "SYNTAX for INPUT_DATA" at L<MCE::Core|MCE::Core>.
+
+   mce_loop { $_ } make_iterator(10, 30, 2);
+
+=item MCE::Loop->run ( sub { code }, list )
 
 =item mce_loop { code } list
 
-Input data can be specified using a list or passing a reference to an array.
+Input data can be defined using a list.
 
    mce_loop { $_ } 1..1000;
    mce_loop { $_ } [ 1..1000 ];
 
+=item MCE::Loop->run_file ( sub { code }, file )
+
 =item mce_loop_f { code } file
 
 The fastest of these is the /path/to/file. Workers communicate the next offset
@@ -500,11 +621,13 @@ position among themselves without any interaction from the manager process.
    mce_loop_f { $_ } $file_handle;
    mce_loop_f { $_ } \$scalar;
 
-=item mce_loop_s { code } sequence
+=item MCE::Loop->run_seq ( sub { code }, $beg, $end [, $step, $fmt ] )
 
-Sequence can be specified as a list, an array reference, or a hash reference.
+=item mce_loop_s { code } $beg, $end [, $step, $fmt ]
+
+Sequence can be defined as a list, an array reference, or a hash reference.
 The functions require both begin and end values to run. Step and format are
-optional. The format is passed to sprintf (% can be omitted below).
+optional. The format is passed to sprintf (% may be omitted below).
 
    my ($beg, $end, $step, $fmt) = (10, 20, 0.1, "%4.1f");
 
@@ -517,6 +640,58 @@ optional. The format is passed to sprintf (% can be omitted below).
 
 =back
 
+The sequence engine can compute 'begin' and 'end' items only, for the chunk,
+and not the items in between (hence boundaries only). This option applies
+to sequence only and has no effect when chunk_size equals 1.
+
+The time to run is 0.006s below. This becomes 0.827s without the bounds_only
+option due to computing all items in between, thus creating a very large
+array. Basically, specify bounds_only => 1 when boundaries is all you need
+for looping inside the block; e.g. Monte Carlo simulations.
+
+Time was measured using 1 worker to emphasize the difference.
+
+   use MCE::Loop;
+
+   MCE::Loop::init {
+      max_workers => 1, chunk_size => 1_250_000,
+      bounds_only => 1
+   };
+
+   ## For sequence, the input scalar $_ points to $chunk_ref
+   ## when chunk_size > 1, otherwise $chunk_ref->[0].
+   ##
+   ## mce_loop_s {
+   ##    my $begin = $_->[0]; my $end = $_->[-1];
+   ##
+   ##    for ($begin .. $end) {
+   ##       ...
+   ##    }
+   ##
+   ## } 1, 10_000_000;
+
+   mce_loop_s {
+      my ($mce, $chunk_ref, $chunk_id) = @_;
+      ## $chunk_ref contains 2 items, not 1_250_000
+
+      my $begin = $chunk_ref->[ 0];
+      my $end   = $chunk_ref->[-1];   ## or $chunk_ref->[1]
+
+      MCE->printf("%7d .. %8d\n", $begin, $end);
+
+   } 1, 10_000_000;
+
+   -- Output
+
+         1 ..  1250000
+   1250001 ..  2500000
+   2500001 ..  3750000
+   3750001 ..  5000000
+   5000001 ..  6250000
+   6250001 ..  7500000
+   7500001 ..  8750000
+   8750001 .. 10000000
+
 =head1 GATHERING DATA
 
 Unlike MCE::Map where gather and output order are done for you automatically,
@@ -528,8 +703,7 @@ the gather method is used to have results sent back to the manager process.
    my @a = mce_loop { MCE->gather($_ * 2) } 1..100;
    print "@a\n\n";
 
-   ## However, one can store to a hash by gathering 2 items per
-   ## each gather call (key, value).
+   ## Outputs to a hash instead (key, value).
    my %h1 = mce_loop { MCE->gather($_, $_ * 2) } 1..100;
    print "@h1{1..100}\n\n";
 
@@ -541,7 +715,7 @@ The gather method can be called multiple times within the block unlike return
 which would leave the block. Therefore, think of gather as yielding results
 immediately to the manager process without actually leaving the block.
 
-   use MCE::Loop chunk_size => 1, max_workers => '3';
+   use MCE::Loop chunk_size => 1, max_workers => 3;
 
    my @hosts = qw(
       hosta hostb hostc hostd hoste
@@ -594,12 +768,10 @@ immediately to the manager process without actually leaving the block.
    Worker 1: Hello from hoste
    Exit status: 0
 
-Serialization is automatic behind the scene. The following uses an anoymous
-array containing 3 elements when gathering data. It's your choice. Obviously,
-calling gather once will be more efficient for IPC.
+The following uses an anonymous array containing 3 elements when gathering
+data. Serialization is automatic behind the scene.
 
    my %h3 = mce_loop {
-
       ...
 
       MCE->gather($host, [$output, $error, $status]);
@@ -612,32 +784,32 @@ calling gather once will be more efficient for IPC.
       print "Exit status: ", $h3{$host}->[2], "\n\n";
    }
 
-Perhaps, you want more control with gather such as appending to an array while
-retaining output order. Although MCE::Map comes to mind, some folks want "full"
-control. And here we go... but this time around in chunking style... :)
+Although MCE::Map comes to mind, one may want additional control when
+gathering data such as retaining output order.
 
-The two options passed to MCE::Loop are optional as they default to 'auto'. The
-beauty of chunking data is that IPC occurs once per chunk versus once per item.
-Although IPC is quite fast, chunking becomes beneficial the larger the data
-becomes. Thus, the reason for the demonstration below.
-
-   use MCE::Loop chunk_size => 'auto', max_workers => 'auto';
-
-   my (@m2, %_tmp); my $_order_id = 1;
+   use MCE::Loop;
 
    sub preserve_order {
-      $_tmp{ (shift) } = \@_;
+      my %tmp; my $order_id = 1; my $gather_ref = $_[0];
 
-      while (1) {
-         last unless exists $_tmp{$_order_id};
-         push @m2, @{ $_tmp{$_order_id} };
-         delete $_tmp{$_order_id++};
-      }
+      return sub {
+         $tmp{ (shift) } = \@_;
 
-      return;
+         while (1) {
+            last unless exists $tmp{$order_id};
+            push @{ $gather_ref }, @{ delete $tmp{$order_id++} };
+         }
+
+         return;
+      };
    }
 
-   MCE::Loop::init { gather => \&preserve_order };
+   my @m2;
+
+   MCE::Loop::init {
+      chunk_size => 'auto', max_workers => 'auto',
+      gather => preserve_order(\@m2)
+   };
 
    mce_loop {
       my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
@@ -651,22 +823,24 @@ becomes. Thus, the reason for the demonstration below.
 
    } 1..100000;
 
+   MCE::Loop::finish;
+
    print scalar @m2, "\n";
 
-That was fast and fun. All 5 models support 'auto' for chunk_size whereas the
-core API doesn't. Think of the models as the basis for providing JIT for MCE.
-They create the instance and tune max_workers plus chunk_size automatically
-irregardless of the hardware being run on.
+All 6 models support 'auto' for chunk_size unlike the Core API. Think of the
+models as the basis for providing JIT for MCE. They create the instance, tune
+max_workers, and tune chunk_size automatically regardless of the hardware.
 
-The following does the same thing using the core API.
+The following does the same thing using the Core API.
 
    use MCE;
 
-   ...
+   sub preserve_order {
+      ...
+   }
 
    my $mce = MCE->new(
       max_workers => 'auto', chunk_size => 8000,
-      gather => \&preserve_order,
 
       user_func => sub {
          my @a; my ($mce, $chunk_ref, $chunk_id) = @_;
@@ -680,19 +854,24 @@ The following does the same thing using the core API.
       }
    );
 
-   $mce->process([1..100000]);
+   my @m2;
 
-   ...
+   $mce->process({ gather => preserve_order(\@m2) }, [1..100000]);
+   $mce->shutdown;
+
+   print scalar @m2, "\n";
 
 =head1 MANUAL SHUTDOWN
 
-=over 2
+=over 3
 
-=item finish
+=item MCE::Loop->finish
 
-MCE workers remain persistent as much as possible after running. Shutdown
-occurs when the script exits. One can manually shutdown MCE by simply calling
-finish after running. This resets the MCE instance.
+=item MCE::Loop::finish
+
+Workers remain persistent as much as possible after running. Shutdown occurs
+automatically when the script terminates. Call finish when workers are no
+longer needed.
 
    use MCE::Loop;
 
@@ -708,19 +887,11 @@ finish after running. This resets the MCE instance.
 
 =head1 INDEX
 
-L<MCE>
+L<MCE|MCE>
 
 =head1 AUTHOR
 
 Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
-=head1 LICENSE
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
 =cut
 
