@@ -1,6 +1,6 @@
 ###############################################################################
 ## ----------------------------------------------------------------------------
-## MCE::Signal - Temporary directory creation/cleanup & signal handling.
+## MCE::Signal - Temporary directory creation/cleanup and signal handling.
 ##
 ###############################################################################
 
@@ -9,23 +9,39 @@ package MCE::Signal;
 use strict;
 use warnings;
 
-use Fcntl qw( :flock O_RDONLY );
-use base qw( Exporter );
+no warnings qw( threads recursion uninitialized );
 
-our $VERSION = '1.499_005'; $VERSION = eval $VERSION;
+our $VERSION = '1.698_001';
+
+## no critic (BuiltinFunctions::ProhibitStringyEval)
 
-our ($has_threads, $main_proc_id, $prog_name);
 our ($display_die_with_localtime, $display_warn_with_localtime);
+our ($main_proc_id, $prog_name, $tmp_dir);
+
+use Carp ();
 
 BEGIN {
-   require File::Path;
-
    $main_proc_id =  $$;
    $prog_name    =  $0;
    $prog_name    =~ s{^.*[\\/]}{}g;
+   $prog_name    =  'perl' if ($prog_name eq '-e' || $prog_name eq '-');
+
+   if ($^O eq 'MSWin32' && !defined $threads::VERSION) {
+      local $@; local $SIG{__DIE__} = \&_NOOP;
+      eval 'use threads; use threads::shared';
+   }
+   elsif (defined $threads::VERSION) {
+      unless (defined $threads::shared::VERSION) {
+         local $@; local $SIG{__DIE__} = \&_NOOP;
+         eval 'use threads::shared';
+      }
+   }
 }
 
-our $tmp_dir = undef;
+use File::Path ();
+use Time::HiRes qw( sleep time );
+use Fcntl qw( :flock );
+use base qw( Exporter );
 
 our @EXPORT_OK = qw( $tmp_dir sys_cmd stop_and_exit );
 our %EXPORT_TAGS = (
@@ -33,16 +49,19 @@ our %EXPORT_TAGS = (
    tmp_dir => [ qw( $tmp_dir ) ]
 );
 
+sub _NOOP {}
+
 ###############################################################################
 ## ----------------------------------------------------------------------------
 ## Process import, export, & module arguments.
 ##
 ###############################################################################
 
-sub _croak { require Carp; goto &Carp::croak; }
-sub _usage { _croak "MCE::Signal error: $_[0] is not a valid option"; }
-sub _flag  { 1; }
+sub _croak { $\ = undef; goto &Carp::croak }
+sub _usage { _croak "MCE::Signal error: ($_[0]) is not a valid option" }
+sub _flag  { 1 }
 
+my $_is_MSWin32   = ($^O eq 'MSWin32') ? 1 : 0;
 my $_keep_tmp_dir = 0;
 my $_no_sigmsg    = 0;
 my $_no_kill9     = 0;
@@ -50,8 +69,7 @@ my $_loaded;
 
 sub import {
 
-   my $class = shift;
-   return if ($_loaded++);
+   my $_class = shift; return if ($_loaded++);
 
    my @_export_args = ();
    my $_no_setpgrp  = 0;
@@ -74,23 +92,18 @@ sub import {
    }
 
    local $Exporter::ExportLevel = 1;
-   Exporter::import($class, @_export_args);
-
- # ## MCE no longer calls setpgrp by default as of MCE 1.405.
- # ## Sets the current process group for the current process.
- # setpgrp(0,0) if ($_no_setpgrp == 0 && $^O ne 'MSWin32');
+   Exporter::import($_class, @_export_args);
 
    ## Sets the current process group for the current process.
    setpgrp(0,0) if ($_setpgrp == 1 && $^O ne 'MSWin32');
 
-   my ($_tmp_dir_base, $_count);
+   my ($_tmp_dir_base, $_count); $_count = 0;
 
-   if (exists $ENV{TEMP}) {
-      if ($^O eq 'MSWin32') {
+   if (exists $ENV{TEMP} && -d $ENV{TEMP} && -w $ENV{TEMP}) {
+      if ($_is_MSWin32) {
          $_tmp_dir_base = $ENV{TEMP} . '/mce';
          mkdir $_tmp_dir_base unless (-d $_tmp_dir_base);
-      }
-      else {
+      } else {
          $_tmp_dir_base = $ENV{TEMP};
       }
    }
@@ -99,15 +112,17 @@ sub import {
          ? '/dev/shm' : '/tmp';
    }
 
-   _croak("MCE::Signal::import: '$_tmp_dir_base' is not writeable")
-      unless (-w $_tmp_dir_base);
+   _croak("Error: MCE::Signal: ($_tmp_dir_base) is not writeable")
+      if (! exists $ENV{'MOBASTARTUPDIR'} && ! -w $_tmp_dir_base);
 
-   $_count  = 0;
-   $tmp_dir = "$_tmp_dir_base/$prog_name.$$.$_count";
+   ## Remove tainted'ness from $tmp_dir.
+   ($tmp_dir) = "$_tmp_dir_base/$prog_name.$$.$_count" =~ /(.*)/;
 
-   while ( !(mkdir "$tmp_dir", 0770) ) {
-      $tmp_dir = "$_tmp_dir_base/$prog_name.$$." . (++$_count);
+   while ( !(mkdir $tmp_dir, 0770) ) {
+      ($tmp_dir) = ("$_tmp_dir_base/$prog_name.$$.".(++$_count)) =~ /(.*)/;
    }
+
+   return;
 }
 
 ###############################################################################
@@ -116,7 +131,12 @@ sub import {
 ##
 ###############################################################################
 
-our ($mce_sess_dir_ref, $mce_spawned_ref);
+my ($_mce_sess_dir_ref, $_mce_spawned_ref);
+
+sub _set_session_vars {
+   ($_mce_sess_dir_ref, $_mce_spawned_ref) = @_;
+   return;
+}
 
 ## Set traps to catch signals.
 $SIG{XCPU} = \&stop_and_exit if (exists $SIG{XCPU});   ## UNIX SIG 24
@@ -129,10 +149,10 @@ $SIG{QUIT} = \&stop_and_exit;                          ## UNIX SIG  3
 $SIG{TERM} = \&stop_and_exit;                          ## UNIX SIG 15
 
 ## For a more reliable MCE, $SIG{CHLD} is set to 'DEFAULT'. MCE handles
-## the reaping of it's children, especially when running multiple MCEs
+## the reaping of its children, especially when running multiple MCEs
 ## simultaneously.
 ##
-$SIG{CHLD} = 'DEFAULT' if ($^O ne 'MSWin32');
+$SIG{CHLD} = 'DEFAULT' unless ($_is_MSWin32);
 
 ###############################################################################
 ## ----------------------------------------------------------------------------
@@ -144,7 +164,7 @@ END {
    my $_exit_status = $?;
 
    MCE::Signal->_shutdown_mce($_exit_status)
-      if (defined $mce_spawned_ref);
+      if (defined $_mce_spawned_ref);
 
    MCE::Signal->stop_and_exit($_exit_status)
       if ($$ == $main_proc_id);
@@ -167,15 +187,20 @@ END {
 sub sys_cmd {
 
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
-   _croak("MCE::Signal::sys_cmd: no arguments was specified") if (@_ == 0);
+
+   _croak('MCE::Signal::sys_cmd: no arguments were specified') if (@_ == 0);
 
    my $_status = system(@_);
    my $_sig_no = $_status & 127;
    my $_exit_status = $_status >> 8;
 
-   ## Kill this process if command caught SIGINT or SIGQUIT.
-   kill('INT',  $$) if $_sig_no == 2;
-   kill('QUIT', $$) if $_sig_no == 3;
+   ## Kill the process group if command caught SIGINT or SIGQUIT.
+
+   kill('INT',  $main_proc_id, ($_is_MSWin32 ? -$$ : -getpgrp))
+      if $_sig_no == 2;
+
+   kill('QUIT', $main_proc_id, ($_is_MSWin32 ? -$$ : -getpgrp))
+      if $_sig_no == 3;
 
    return $_exit_status;
 }
@@ -189,136 +214,130 @@ sub sys_cmd {
 ##
 ###############################################################################
 
-{
-   my $_handler_cnt = 0;
+my %_sig_name_lkup = map { $_ => 1 } qw(
+   __DIE__ __WARN__ HUP INT PIPE QUIT TERM CHLD XCPU XFSZ
+);
 
-   my %_sig_name_lkup = map { $_ => 1 } qw(
-      __DIE__ __WARN__ HUP INT PIPE QUIT TERM CHLD XCPU XFSZ
-   );
+my $_handler_cnt : shared = 0;
 
-   sub stop_and_exit {
+sub stop_and_exit {
 
-      shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
+   shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
-      my $_sig_name    = $_[0] || 0;
-      my $_exit_status = $?;
-      my $_is_sig      = 0;
+   my $_is_main_proc = ($$ == $main_proc_id);
+   my $_sig_name     = $_[0] || 0;
+   my $_exit_status  = $?;
+   my $_is_sig       = 0;
 
-      if (exists $_sig_name_lkup{$_sig_name}) {
-         $mce_spawned_ref = undef;
-         $SIG{$_sig_name} = sub { };
-         $_exit_status = $_is_sig = 1;
-      }
-      else {
-         $_exit_status = $_sig_name if ($_sig_name ne '0');
-      }
+   if (exists $_sig_name_lkup{$_sig_name}) {
+      $_mce_spawned_ref = undef;
+      $SIG{$_sig_name}  = \&_NOOP;
+      $_exit_status     = $_is_sig = 1;
+   } else {
+      $_exit_status = $_sig_name if ($_sig_name ne '0');
+   }
 
-      $SIG{TERM} = sub { } if ($_sig_name ne 'TERM');
-      $SIG{__DIE__} = $SIG{__WARN__} = sub { };
+   $SIG{  INT  } = \&_NOOP if ($_sig_name ne 'INT');
+   $SIG{__DIE__} = $SIG{__WARN__} = \&_NOOP;
 
-      ## ----------------------------------------------------------------------
+   ## -------------------------------------------------------------------------
 
-      ## For main thread / parent process.
-      if ($$ == $main_proc_id) {
+   ## For the main process.
+   if ($_is_main_proc) {
+      lock $_handler_cnt if $INC{'threads/shared.pm'};
 
-         $_handler_cnt += 1;
+      if (++$_handler_cnt == 1) {
+         open my $_FH, '>>', "$tmp_dir/stopped"; close $_FH;
+         local $\ = undef;
 
-         if ($_handler_cnt == 1 && ! -e "$tmp_dir/stopped") {
-            open my $_FH, "> $tmp_dir/stopped"; close $_FH;
+         ## Display message and kill process group if signaled.
+         if ($_is_sig == 1) {
+            my $_err_msg = undef;
 
-            local $\ = undef;
-
-            ## Display message and kill process group if signaled.
-            if ($_is_sig == 1) {
-               my $_err_msg = undef;
-
-               if ($_sig_name eq 'XCPU') {
-                  $_err_msg = "exceeded CPU time limit, exiting";
-               }
-               elsif ($_sig_name eq 'XFSZ') {
-                  $_err_msg = "exceeded file size limit, exiting";
-               }
-               elsif ($_sig_name eq 'TERM' && -f "$tmp_dir/died") {
-                  $_err_msg = "caught signal '__DIE__', exiting";
-               }
-               elsif ($_sig_name eq '__DIE__') {
-                  $_err_msg = "caught signal '__DIE__', exiting";
-               }
-               elsif ($_sig_name ne 'PIPE') {
-                  $_err_msg = "caught signal '$_sig_name', exiting";
-               }
-
-               ## Display error message.
-               print STDERR "\n## $prog_name: $_err_msg\n"
-                  if ($_err_msg && $_no_sigmsg == 0);
-
-               open my $_FH, "> $tmp_dir/killed"; close $_FH;
-
-               ## Signal process group to terminate.
-               kill('TERM', -$$);
-
-               ## Pause a bit.
-               if ($_sig_name ne 'PIPE') {
-                  select(undef, undef, undef, 0.066) for (1..3);
-               }
+            if ($_sig_name eq 'XCPU') {
+               $_err_msg = 'exceeded CPU time limit, exiting';
+            } elsif ($_sig_name eq 'XFSZ') {
+               $_err_msg = 'exceeded file size limit, exiting';
+            } elsif ($_sig_name eq 'INT' && -f "$tmp_dir/died") {
+               $_err_msg = 'caught signal (__DIE__), exiting';
+            } elsif ($_sig_name eq '__DIE__') {
+               $_err_msg = 'caught signal (__DIE__), exiting';
+            } elsif ($_sig_name ne 'PIPE') {
+               $_err_msg = "caught signal ($_sig_name), exiting";
+            }
+            if ($_err_msg && $_no_sigmsg == 0) {
+               print {*STDERR} "\n## $prog_name: $_err_msg\n";
             }
 
-            ## Remove temp directory.
-            if (defined $tmp_dir && $tmp_dir ne '' && -d $tmp_dir) {
+            open my $_FH, '>', "$tmp_dir/killed"; close $_FH;
 
-               if (defined $mce_sess_dir_ref) {
-                  foreach my $_sess_dir (keys %{ $mce_sess_dir_ref }) {
-                     File::Path::rmtree($_sess_dir);
-                     delete $mce_sess_dir_ref->{$_sess_dir};
-                  }
-               }
+            kill('INT', $_is_MSWin32 ? -$$ : -getpgrp);
 
-               if ($_keep_tmp_dir == 1) {
-                  print STDERR "$prog_name: saved tmp_dir = $tmp_dir\n";
-               }
-               else {
-                  if ($tmp_dir ne '/tmp' && $tmp_dir ne '/var/tmp') {
-                     File::Path::rmtree($tmp_dir);
-                  }
-               }
-
-               $tmp_dir = undef;
-            }
-
-            ## Signal process group to die.
-            if ($_is_sig == 1) {
-               print STDERR "\n"
-                  if ($_sig_name ne 'PIPE' && $_no_sigmsg == 0);
-
-               kill('KILL', -$$, $main_proc_id)
-                  if ($_sig_name eq 'PIPE' || $_no_kill9 == 0);
+            ## Pause a bit.
+            if ($_sig_name eq 'PIPE') {
+               sleep 0.015 for (1..2);
+            } else {
+               sleep 0.065 for (1..3);
             }
          }
+
+         ## Otherwise, shutdown MCE::Shared if running.
+         elsif ($INC{'MCE/Shared/Server.pm'}) {
+            MCE::Shared::Server::_shutdown();
+         }
+
+         ## Remove temp directory.
+         if (defined $tmp_dir && $tmp_dir ne '' && -d $tmp_dir) {
+            if (defined $_mce_sess_dir_ref) {
+               for my $_sess_dir (keys %{ $_mce_sess_dir_ref }) {
+                  File::Path::rmtree($_sess_dir);
+                  delete $_mce_sess_dir_ref->{$_sess_dir};
+               }
+            }
+            if ($_keep_tmp_dir == 1) {
+               print {*STDERR} "$prog_name: saved tmp_dir = $tmp_dir\n";
+            } elsif ($tmp_dir ne '/tmp' && $tmp_dir ne '/var/tmp') {
+               File::Path::rmtree($tmp_dir);
+            }
+            $tmp_dir = undef;
+         }
+
+         ## Signal process group to die.
+         if ($_is_sig == 1) {
+            if ($_sig_name ne 'PIPE' && $_no_sigmsg == 0) {
+               print {*STDERR} "\n";
+            }
+            ($_no_kill9 == 1 || $_sig_name eq 'PIPE')
+               ? kill('INT', $_is_MSWin32 ? -$$ : -getpgrp)
+               : kill('KILL', -$$, $main_proc_id);
+         }
       }
+   }
 
-      ## ----------------------------------------------------------------------
-
-      ## For child processes.
-      if ($$ != $main_proc_id && $_is_sig == 1 && -d $tmp_dir) {
+   ## Otherwise, for child processes.
+   elsif ($_is_sig == 1 && -d $tmp_dir) {
+      if (++$_handler_cnt == 1) {
 
          ## Obtain lock.
          open my $CHILD_LOCK, '+>>', "$tmp_dir/child.lock";
          flock $CHILD_LOCK, LOCK_EX;
 
-         $_handler_cnt += 1;
+         ## Notify the main process that I've died.
+         if ($_sig_name eq '__DIE__' && ! -f "$tmp_dir/died") {
+            local $@; eval {
+               open my $_FH, '>>', "$tmp_dir/died"; close $_FH;
+            };
+         }
 
          ## Signal process group to terminate.
-         if ($_handler_cnt == 1) {
-
-            ## Notify the main process that I've died.
-            if ($_sig_name eq '__DIE__' && ! -f "$tmp_dir/died") {
-               open my $_FH, "> $tmp_dir/died"; close $_FH;
-            }
-
-            ## Signal process group to terminate.
-            if (! -f "$tmp_dir/killed" && ! -f "$tmp_dir/stopped") {
-               open my $_FH, "> $tmp_dir/killed"; close $_FH;
-               kill('TERM', -$$, $main_proc_id);
+         if (! -f "$tmp_dir/killed" && ! -f "$tmp_dir/stopped") {
+            local $@; eval {
+               open my $_FH, '>>', "$tmp_dir/killed"; close $_FH;
+            };
+            if ($_sig_name eq 'PIPE') {
+               kill('PIPE', $main_proc_id, -$$);
+            } else {
+               kill('INT', $main_proc_id, -$$);
             }
          }
 
@@ -326,17 +345,16 @@ sub sys_cmd {
          flock $CHILD_LOCK, LOCK_UN;
          close $CHILD_LOCK;
       }
-
-      ## ----------------------------------------------------------------------
-
-      ## Exit thread/process with status.
-      if ($_is_sig == 1 && $_no_kill9 == 0) {
-         select(undef, undef, undef, 0.066) for (1..6);
-      }
-
-      threads->exit($_exit_status) if ($has_threads && threads->can('exit'));
-      CORE::exit($_exit_status);
    }
+
+   ## -------------------------------------------------------------------------
+
+   ## Exit with status.
+   if ($_is_sig == 1 && $_no_kill9 == 0) {
+      sleep 0.065 for (1..6);
+   }
+
+   CORE::exit($_exit_status);
 }
 
 ###############################################################################
@@ -351,21 +369,27 @@ sub _shutdown_mce {
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
    my $_exit_status = $_[0] || $?;
 
-   if (defined $mce_spawned_ref) {
-      my $_tid = ($has_threads) ? threads->tid() : '';
-      $_tid = '' unless defined $_tid;
+   if (defined $_mce_spawned_ref) {
+      my $_tid = ($INC{'threads.pm'}) ? threads->tid() : '';
+         $_tid = '' unless defined $_tid;
 
-      foreach my $_mce_sid (keys %{ $mce_spawned_ref }) {
-         if ($mce_spawned_ref->{$_mce_sid}->wid()) {
-            $mce_spawned_ref->{$_mce_sid}->exit($_exit_status);
+      for my $_mce_sid (keys %{ $_mce_spawned_ref }) {
+         if ($_mce_spawned_ref->{$_mce_sid}->wid()) {
+            if ($_mce_spawned_ref->{$_mce_sid}->pid() == $$) {
+               $_mce_spawned_ref->{$_mce_sid}->exit($_exit_status);
+            }
          }
          else {
-            $mce_spawned_ref->{$_mce_sid}->shutdown()
-               if ($_mce_sid =~ /\A$$\.$_tid\./);
+            if ($_mce_sid =~ /\A$$\.$_tid\./) {
+               $_mce_spawned_ref->{$_mce_sid}->shutdown();
+            }
          }
-         delete $mce_spawned_ref->{$_mce_sid};
+
+         delete $_mce_spawned_ref->{$_mce_sid};
       }
    }
+
+   return;
 }
 
 ###############################################################################
@@ -378,24 +402,41 @@ sub _die_handler {
 
    shift @_ if (defined $_[0] && $_[0] eq 'MCE::Signal');
 
-   CORE::die(@_) if $^S;      ## Direct to CORE::die if executing an eval
+   if (!defined $^S || $^S) {
+      if ( ($INC{'threads.pm'} && threads->tid() != 0) ||
+            $ENV{'PERL_IPERL_RUNNING'}
+      ) {
+         # thread env or running inside IPerl, check stack trace
+         my $_t = Carp::longmess(); $_t =~ s/\teval [^\n]+\n$//;
+
+         if ( $_t =~ /^(?:[^\n]+\n){1,7}\teval / ||
+              $_t =~ /\n\teval [^\n]+\n\t(?:eval|Try)/ )
+         {
+            CORE::die(@_);
+         }
+      }
+      else {
+         # normal env, trust $^S
+         CORE::die(@_);
+      }
+   }
 
    local $\ = undef;
 
    ## Set $MCE::Signal::display_die_with_localtime = 1;
    ## when wanting the output to contain the localtime.
 
-   if ($MCE::Signal::display_die_with_localtime) {
-      my $_time_stamp = localtime();
-      print STDERR "## $_time_stamp: $prog_name: ERROR:\n", $_[0];
-   }
-   else {
-      print STDERR $_[0];
+   if (defined $_[0]) {
+      if ($MCE::Signal::display_die_with_localtime) {
+         my $_time_stamp = localtime;
+         print {*STDERR} "## $_time_stamp: $prog_name: ERROR:\n", $_[0];
+      }
+      else {
+         print {*STDERR} $_[0];
+      }
    }
 
    MCE::Signal->stop_and_exit('__DIE__');
-
-   CORE::exit;
 }
 
 sub _warn_handler {
@@ -407,7 +448,8 @@ sub _warn_handler {
    return if (
       $_[0] =~ /^A thread exited while \d+ threads were running/ ||
       $_[0] =~ /^Attempt to free unreferenced scalar/            ||
-      $_[0] =~ /^Perl exited with active threads/
+      $_[0] =~ /^Perl exited with active threads/                ||
+      $_[0] =~ /^Thread \d+ terminated abnormally/
    );
 
    local $\ = undef;
@@ -415,13 +457,17 @@ sub _warn_handler {
    ## Set $MCE::Signal::display_warn_with_localtime = 1;
    ## when wanting the output to contain the localtime.
 
-   if ($MCE::Signal::display_warn_with_localtime) {
-      my $_time_stamp = localtime();
-      print STDERR "## $_time_stamp: $prog_name: WARNING:\n", $_[0];
-   }
-   else {
-      print STDERR $_[0];
+   if (defined $_[0]) {
+      if ($MCE::Signal::display_warn_with_localtime) {
+         my $_time_stamp = localtime;
+         print {*STDERR} "## $_time_stamp: $prog_name: WARNING:\n", $_[0];
+      }
+      else {
+         print {*STDERR} $_[0];
+      }
    }
+
+   return;
 }
 
 1;
@@ -436,11 +482,11 @@ __END__
 
 =head1 NAME
 
-MCE::Signal - Temporary directory creation/cleanup & signal handling
+MCE::Signal - Temporary directory creation/cleanup and signal handling
 
 =head1 VERSION
 
-This document describes MCE::Signal version 1.499_005
+This document describes MCE::Signal version 1.698_001
 
 =head1 SYNOPSIS
 
@@ -457,8 +503,8 @@ receiving said signals call stop_and_exit, which signals all workers to
 terminate, removes the temporary directory unless -keep_tmp_dir is specified,
 and terminates itself.
 
-The location of temp dir resides under $ENV{TEMP} if defined, otherwise
-/dev/shm if writeable and -use_dev_shm is specified, or /tmp.
+The location of the temp directory resides under $ENV{TEMP} if defined,
+otherwise /dev/shm if writeable and -use_dev_shm is specified, or /tmp.
 
 The temp dir resides under $ENV{TEMP}/mce/ for native Perl on Microsoft
 Windows.
@@ -466,9 +512,10 @@ Windows.
 As of MCE 1.405, MCE::Signal no longer calls setpgrp by default. Pass the
 -setpgrp option to MCE::Signal to call setpgrp.
 
- ## Running MCE through Daemon::Control requires setpgrp to be called.
+ ## Running MCE through Daemon::Control requires setpgrp to be called
+ ## for MCE releases 1.511 and below.
 
- use MCE::Signal qw(-setpgrp);
+ use MCE::Signal qw(-setpgrp);   ## Not necessary for MCE 1.512 and above
  use MCE;
 
 The following are available arguments and their meanings.
@@ -483,11 +530,13 @@ The following are available arguments and their meanings.
 
  -setpgrp          - Calls setpgrp to set the process group for the process
 
-                     Specify this option to ensure all workers terminate
-                     when reading STDIN like so:
+                     This option ensures all workers terminate when reading
+                     STDIN for MCE releases 1.511 and below.
+
                         cat big_input_file | ./mce_script.pl | head -10
 
                      This works fine without the -setpgrp option:
+
                         ./mce_script.pl < big_input_file | head -10
 
 Nothing is exported by default. Exportable are 1 variable and 2 subroutines.
@@ -498,27 +547,33 @@ Nothing is exported by default. Exportable are 1 variable and 2 subroutines.
 
 =head2 stop_and_exit ( [ $exit_status | $signal ] )
 
- ## Stops execution, removes temp directory and exits the entire
- ## application. Pass 'TERM' if wanting to terminate a spawned or
- ## running MCE state.
+Stops execution, removes temp directory, and exits the entire application.
+Pass 'INT' to terminate a spawned or running MCE session.
 
  MCE::Signal::stop_and_exit(1);
- MCE::Signal::stop_and_exit('TERM');
+ MCE::Signal::stop_and_exit('INT');
 
 =head2 sys_cmd ( $command )
 
- ## Execute command and return the actual exit status. The calling
- ## process is also signaled if command caught SIGINT or SIGQUIT.
+The system function in Perl ignores SIGNINT and SIGQUIT. These 2 signals are
+sent to the command being executed via system() but not back to the underlying
+Perl script. For this reason, sys_cmd was added to MCE::Signal.
 
- my $exit_status = MCE::Signal::sys_cmd($command);
+ ## Execute command and return the actual exit status. The perl script
+ ## is also signaled if command caught SIGINT or SIGQUIT.
+
+ use MCE::Signal qw(sys_cmd);   ## Include before MCE
+ use MCE;
+
+ my $exit_status = sys_cmd($command);
 
 =head1 EXAMPLES
 
- ## Creates tmp_dir under /tmp
+ ## Creates tmp_dir under $ENV{TEMP} if defined, otherwise /tmp
  use MCE::Signal;
  use MCE::Signal qw( :all );
 
- ## Attempt to create tmp_dir under /dev/shm, otherwise under /tmp
+ ## Attempt to create tmp_dir under /dev/shm if writable
  use MCE::Signal qw( -use_dev_shm );
 
  ## Keep tmp_dir after script terminates
@@ -527,19 +582,11 @@ Nothing is exported by default. Exportable are 1 variable and 2 subroutines.
 
 =head1 INDEX
 
-L<MCE>
+L<MCE|MCE>
 
 =head1 AUTHOR
 
 Mario E. Roy, S<E<lt>marioeroy AT gmail DOT comE<gt>>
 
-=head1 LICENSE
-
-This program is free software; you can redistribute it and/or modify it
-under the terms of either: the GNU General Public License as published
-by the Free Software Foundation; or the Artistic License.
-
-See L<http://dev.perl.org/licenses/> for more information.
-
 =cut
 
